unit SpTBXReg;

{==============================================================================
Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND,
either express or implied.
The initial developer of this code is Robert Lee.

Requirements:
  - Jordan Russell's Toolbar 2000
    http://www.jrsoftware.org
  - Alex Denisov TBX
    http://g32.org
==============================================================================}

interface

{$I TB2Ver.inc}

uses
  Windows, Classes, Controls, SysUtils, Graphics, ImgList, Dialogs,
  {$IFDEF JR_D6} DesignIntf, DesignEditors, VCLEditors, {$ELSE} DsgnIntf, {$ENDIF}
  TB2Reg, TB2Toolbar, TB2Item, TBX, TB2DsgnItemEditor,
  SpTBXItem, SpTBXTabs, SpTBXDkPanels;

procedure Register;

implementation

uses
  Forms, TBXThemes, TBXStrEdit, TBXUtils, TypInfo;

//WMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWM
{ THookObj }

type
  THookObj = class
    procedure HookProc(Sender: TTBItemEditForm);
  end;

var SpTBXHook: THookObj;

procedure THookObj.HookProc(Sender: TTBItemEditForm);
var
  TB: TTBToolbar;
  Item: TTBCustomItem;
  NewItem: TTBItem;
  S: string;
  I: Integer;
begin
  // Create our own toolbar in the editor
  TB := TTBToolbar.Create(Sender);
  TB.Top := Sender.Height;
  TB.Parent := Sender;
  TB.Align := alTop;
  TB.Images := Sender.ToolbarItems.SubMenuImages;
  TB.ShowHint := True;
  TB.Name := 'SpTBXToolbar';

  for I := 0 to Sender.MoreMenu.Count - 1 do
  begin
    Item := Sender.MoreMenu.Items[I];
    if Item is TTBCustomItem then
    begin
      S := TTBCustomItemClass(Item.Tag).ClassName;
      if StrLComp(PChar(S), 'TSpTBX', 6) = 0 then
      begin
        NewItem := TTBItem.Create(TB);
        TB.Items.Add(NewItem);
        NewItem.Caption := Item.Caption;
        NewItem.ImageIndex := Item.ImageIndex;
        NewItem.Tag := Item.Tag;
        NewItem.Hint := S;
        NewItem.OnClick := Item.OnClick;
      end;
    end;
  end;
end;

//WMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWMWM

procedure Register;
begin
  RegisterComponents('SpTBXLib', [TSpTBXDock, TSpTBXMultiDock, TSpTBXToolbar,
    TSpTBXDockablePanel, TSpTBXTabSet, TSpTBXTabControl, TSpTBXStatusBar,
    TSpTBXLabel, TSpTBXCheckBox, TSpTBXRadioButton, TSpTBXButton, 
    TSpTBXProgressBar, TSpTBXPanel, TSpTBXGroupBox]);

  RegisterClasses([TSpTBXTabSheet, TSpTBXCustomTabSet, TSpTBXCustomTabControl]);

  // TSpTBXItem
  RegisterNoIcon([TSpTBXItem]);
  RegisterClasses([TSpTBXItem]);
  TBRegisterItemClass(TSpTBXItem, 'New SpTBX Item', HInstance);
  // TSpTBXSubmenuItem
  RegisterNoIcon([TSpTBXSubmenuItem]);
  RegisterClasses([TSpTBXSubmenuItem]);
  TBRegisterItemClass(TSpTBXSubmenuItem, 'New SpTBX Submenu Item', HInstance);
  // TSpTBXSeparatorItem
  RegisterNoIcon([TSpTBXSeparatorItem]);
  RegisterClasses([TSpTBXSeparatorItem]);
  TBRegisterItemClass(TSpTBXSeparatorItem, 'New SpTBX Separator Item', HInstance);
  // TSpTBXSpacerItem
  RegisterNoIcon([TSpTBXRightAlignSpacerItem]);
  RegisterClasses([TSpTBXRightAlignSpacerItem]);
  TBRegisterItemClass(TSpTBXRightAlignSpacerItem, 'New SpTBX RightAlignSpacer Item', HInstance);
  // TSpTBXLabelItem
  RegisterNoIcon([TSpTBXLabelItem]);
  RegisterClasses([TSpTBXLabelItem]);
  TBRegisterItemClass(TSpTBXLabelItem, 'New SpTBX Label Item', HInstance);
  // TSpTBXThemeGroupItem
  RegisterNoIcon([TSpTBXThemeGroupItem]);
  RegisterClasses([TSpTBXThemeGroupItem]);
  TBRegisterItemClass(TSpTBXThemeGroupItem, 'New SpTBX ThemeGroup Item', HInstance);
  // TSpTBXTabItem
  RegisterNoIcon([TSpTBXTabItem]);
  RegisterClasses([TSpTBXTabItem]);
  TBRegisterItemClass(TSpTBXTabItem, 'New SpTBX Tab Item', HInstance);

  // Register the components editor, the components must implement IItems interface
  RegisterComponentEditor(TSpTBXCompoundItemsControl, TTBItemsEditor);
  RegisterComponentEditor(TSpTBXDockablePanel, TTBItemsEditor);
end;

initialization
  SpTBXHook := THookObj.Create;
  TBUnregisterDsgnEditorHook(SpTBXHook.HookProc);
  TBRegisterDsgnEditorHook(SpTBXHook.HookProc);

finalization
  TBUnregisterDsgnEditorHook(SpTBXHook.HookProc);
  SpTBXHook.Free;

end.
